function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

import Config from '../config-provider';
import { makeEventStream } from './make-event-stream';

import Request from '../service/main/request';
import NetworkerThread from '../service/networker';

import { NetMessage } from '../service/networker/net-message';
import { MTError } from '../error';

import Logger from 'mtproto-logger';
var log = Logger`stream-bus`;

function _ref3(value) {
  return value.isAPI;
}

function _ref4(value) {
  return !value.isAPI;
}

function _ref7(error) {
  return error.code === 500 && error.type === 'AUTH_RESTART';
}

var createStreamBus = ctx => {
  var emitter = Config.rootEmitter(ctx.uid);
  var bus = makeStreamMap(emitter);

  // pushMessage.onValue(log('push message'))

  bus.responseRaw.onValue(log('raw response'));
  bus.responseRaw.onError(log('raw error'));

  bus.incomingMessage.onValue(log('incoming message'));

  var state = ctx.state;

  bus.incomingMessage.observe({
    value(val) {
      var networker = state.threads.get(val.threadID);
      if (networker == null) return;
      log('observer', 'type')(val.message._, networker.dcID);
    }
  });

  bus.newNetworker.observe(networker => {
    log('new networker')(networker);
    state.threads.set(networker.threadID, networker);
  });

  function* _ref2({
    threadID,
    networkerDC,
    newDc,
    error,
    sentMessage,
    message
  }) {
    log('migrate')(message);
    yield ctx.storage.set('dc', newDc);
    // const thread = ctx.state.threads.get(threadID)
    // if (thread == null) return
    if (typeof sentMessage.requestID !== 'string') return;
    var req = ctx.state.requests.get(sentMessage.requestID);
    if (req == null) return;
    req.options.dc = newDc;
    log('migrate', 'req')(req);
    log('on migrate restart')('before end');
    yield ctx.api.invokeNetRequest(req);
    // thread.generateSeqNo(notContentRelated?)
    // message.clone
  }

  bus.migrateError.observe((() => {
    var _ref = _asyncToGenerator(_ref2);

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  })());

  bus.messageIn.onValue(log('message in'));

  var apiOnly = bus.messageIn.filter(_ref3);
  var mtOnly = bus.messageIn.filter(_ref4);

  apiOnly.observe({
    value(val) {
      ctx.state.messages.set(val.msg_id, val);
    }
  });

  function* _ref6(data) {
    log('rpc result')(data);
    ctx.state.messages.delete(data.sentMessage.msg_id);
    ctx.state.requests.delete(data.sentMessage.requestID);
    data.sentMessage.deferred.resolve(data.result);
  }

  bus.rpcResult.observe((() => {
    var _ref5 = _asyncToGenerator(_ref6);

    return function (_x2) {
      return _ref5.apply(this, arguments);
    };
  })());

  bus.rpcError.onValue(log('rpc error'));

  var isAuthRestart = _ref7;

  function* _ref9(data) {
    if (isAuthRestart(data.error)) {
      if (!ctx.state.messages.has(data.message.req_msg_id)) {
        data.sentMessage.deferred.reject(data.error);
        return log('on auth restart')(data.message.req_msg_id, 'req_msg_id not found');
      }
      var msg = ctx.state.messages.get(data.message.req_msg_id);
      if (!msg || !msg.requestID) {
        data.sentMessage.deferred.reject(data.error);
        return log('on auth restart')('msg', msg);
      }
      var req = ctx.state.requests.get(msg.requestID);
      if (!req) {
        data.sentMessage.deferred.reject(data.error);
        return log('on auth restart')('req', req);
      }

      log('on auth restart')(`dc${data.networkerDC}_auth_key`, `dc${data.networkerDC}_server_salt`);
      yield ctx.storage.remove(`dc${data.networkerDC}_auth_key`, `dc${data.networkerDC}_server_salt`);
      log('on auth restart')('before end');
      yield ctx.api.invokeNetRequest(req);
    } else {
      log('rpc', 'unhandled')(data);
      data.sentMessage.deferred.reject(data.error);
    }
  }

  bus.rpcError.observe((() => {
    var _ref8 = _asyncToGenerator(_ref9);

    return function (_x3) {
      return _ref8.apply(this, arguments);
    };
  })());

  bus.netMessage.onValue(message => {
    log('net message')(message);
    var req = ctx.state.requests.get(message.options.requestID);
    log('req')(req);
  });

  bus.netMessage.onValue(log('new request'));

  function* _ref11(netReq) {
    if (state.requests.has(netReq.requestID)) return log('request', 'repeat')(netReq);
    ctx.state.requests.set(netReq.requestID, netReq);
    var dc = netReq.options.dc;
    if (!dc || dc === '@@home') {
      var fromStore = yield ctx.storage.get('dc');
      dc = fromStore ? +fromStore : ctx.defaultDC;
    }
    netReq.options.dc = dc;

    log('request', 'new')(netReq);
  }

  bus.newRequest.observe((() => {
    var _ref10 = _asyncToGenerator(_ref11);

    return function (_x4) {
      return _ref10.apply(this, arguments);
    };
  })());

  return bus;
};

var an = {};

var pushMessageCast = an;
var responseRawCast = an;
var incomingMessageCast = an;
var newNetworkerCast = an;
var migrateErrorCast = an;
var rpcErrorCast = an;
var rpcResultCast = an;

var netMessageCast = an;
var newRequestCast = an;
var messageInCast = an;

function makeStreamMap(emitter) {
  var getter = makeEventStream(emitter);

  var pushMessage = getter('push-message', pushMessageCast);
  var responseRaw = getter('response-raw', responseRawCast);
  var incomingMessage = getter('incoming-message', incomingMessageCast);
  var newNetworker = getter('new-networker', newNetworkerCast);
  var migrateError = getter('migrate-error', migrateErrorCast);
  var rpcError = getter('rpc-error', rpcErrorCast);
  var rpcResult = getter('rpc-result', rpcResultCast);
  var netMessage = getter('net-message', netMessageCast);
  var newRequest = getter('new-request', newRequestCast);
  var messageIn = getter('message-in', messageInCast);

  var streamMap = {
    pushMessage,
    responseRaw,
    incomingMessage,
    newNetworker,
    migrateError,
    rpcError,
    netMessage,
    newRequest,
    messageIn,
    rpcResult
  };

  return streamMap;
}

export default createStreamBus;
//# sourceMappingURL=stream-bus.js.map