function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

import Promise from 'bluebird';
// import UpdatesManager from '../updates'

import propEq from 'ramda/src/propEq';
import has from 'ramda/src/has';

import Logger from 'mtproto-logger';
var debug = Logger`api-manager`;

import Auth from '../authorizer';


import blueDefer from '../../util/defer';
import { chooseServer } from '../dc-configurator';

import KeyManager from '../rsa-keys-manger';
import { AuthKeyError, DcUrlError } from '../../error';

import { bytesFromHex, bytesToHex } from '../../bin';

import { switchErrors } from './error-cases';
import { delayedCall, dTime } from 'mtproto-shared';

import Request from './request';

import Config from '../../config-provider';
import NetworkerThread from '../networker';
import NetRequest from '../main/request';

var baseDcID = 2;

var Ln = (length, obj) => {
  if (obj == null) return false;
  //$FlowIssue
  var res = propEq('length', length, obj);
  return res;
};

export class ApiManager {
  constructor(config, uid) {
    var _this = this;

    this.cache = {
      uploader: {},
      downloader: {},
      auth: {},
      servers: {},
      keysParsed: {}
    };
    this.authPromise = blueDefer();
    this.authBegin = false;
    this.currentDc = 2;
    this.online = false;

    function* _ref2(dcID, userAuth) {
      var fullUserAuth = Object.assign({ dcID }, userAuth);
      yield _this.storage.set('dc', dcID);
      yield _this.storage.set('user_auth', fullUserAuth);
      _this.emit('auth.dc', { dc: dcID, auth: userAuth });
    }

    this.setUserAuth = (() => {
      var _ref = _asyncToGenerator(_ref2);

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    })();

    var {
      server,
      api,
      app: {
        storage,
        publicKeys
      }
    } = config;
    this.uid = uid;
    this.apiConfig = api;
    this.publicKeys = publicKeys;
    this.storage = storage;
    this.serverConfig = server;
    this.chooseServer = chooseServer(this.cache.servers, server);
    var emitter = Config.rootEmitter(this.uid);
    this.on = emitter.on;
    this.emit = emitter.emit;
    this.keyManager = KeyManager(uid, publicKeys, this.cache.keysParsed);
    this.auth = Auth(uid, this.keyManager);

    //$FlowIssue
    this.mtpInvokeApi = this.mtpInvokeApi.bind(this);
    //$FlowIssue
    this.mtpGetNetworker = this.mtpGetNetworker.bind(this);

    // this.updatesManager = UpdatesManager(apiManager, this.TL)
    // apiManager.updates = this.updatesManager

    emitter.on('error.303', newDc => {
      this.authBegin = false;
      this.currentDc = newDc;
    });
  }
  networkSetter(dc, authKey, serverSalt) {
    var networker = new NetworkerThread({
      appConfig: this.apiConfig,
      storage: this.storage,
      chooseServer: this.chooseServer
    }, dc, authKey, serverSalt, this.uid);
    this.cache.downloader[dc] = networker;
    return networker;
  }
  mtpGetNetworker(dcID) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      // const isUpload = options.fileUpload || options.fileDownload
      // const cache = isUpload
      //   ? this.cache.uploader
      //   : this.cache.downloader

      var cache = _this2.cache.downloader;
      if (!dcID) throw new Error('get Networker without dcID');

      if (has(dcID, cache)) return cache[dcID];

      var akk = `dc${dcID}_auth_key`;
      var ssk = `dc${dcID}_server_salt`;

      var dcUrl = _this2.chooseServer(dcID, false);
      if (typeof dcUrl !== 'string') throw new DcUrlError(dcID, dcUrl);

      if (cache[dcID]) return cache[dcID];

      var authKeyHex = yield _this2.storage.get(akk);
      var serverSaltHex = yield _this2.storage.get(ssk);

      // if (!authKeyHex || !serverSaltHex)
      // await this.doAuth()

      if (Ln(512, authKeyHex)) {
        if (!serverSaltHex || serverSaltHex.length !== 16) serverSaltHex = 'AAAAAAAAAAAAAAAA';
        var _authKey = bytesFromHex(authKeyHex);
        var _serverSalt = bytesFromHex(serverSaltHex);

        return _this2.networkSetter(dcID, _authKey, _serverSalt);
      }

      var auth = void 0;
      try {
        auth = yield _this2.auth(dcID, _this2.cache.auth, dcUrl);
      } catch (error) {
        return netError(error);
      }

      var { authKey, serverSalt } = auth;

      yield _this2.storage.set(akk, bytesToHex(authKey));
      yield _this2.storage.set(ssk, bytesToHex(serverSalt));

      return _this2.networkSetter(dcID, authKey, serverSalt);
    })();
  }
  doAuth() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      _this3.authBegin = true;
      try {
        var storedBaseDc = yield _this3.storage.get('dc');
        var baseDc = storedBaseDc || baseDcID;
        var opts = {
          dcID: baseDc,
          createNetworker: true
        };
        var networker = yield _this3.mtpGetNetworker(baseDc);
        var nearestDc = yield networker.wrapApiCall('help.getNearestDc', {}, opts);
        var { nearest_dc, this_dc } = nearestDc;
        if (storedBaseDc == null) {
          yield _this3.storage.set('dc', this_dc);
          // if (nearest_dc !== this_dc) await this.mtpGetNetworker(nearest_dc, {
          //   dcID           : this_dc,
          //   createNetworker: true
          // })
        }
        debug(`nearest Dc`, ` this dc`)(nearestDc, this_dc);
        _this3.emit('testing', { nearestDc, this_dc });
        _this3.authPromise.resolve();
      } catch (err) {
        _this3.authPromise.reject(err);
      }
    })();
  }
  initConnection() {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      if (!isAnyNetworker(_this4)) {
        if (!_this4.authBegin) _this4.doAuth();
        yield _this4.authPromise.promise;
      }
    })();
  }
  mtpInvokeApi(method, params, options = {}) {
    var deferred = blueDefer();
    var netReq = new NetRequest({ method, params }, options, deferred);

    return this.invokeNetRequest(netReq);
  }

  invokeNetRequest(netReq) {
    var _this5 = this;

    function _ref3(error) {
      var err = void 0;
      if (error instanceof Error) err = error;else {
        err = new Error();
        //$FlowIssue
        err.data = error;
      }
      // if (!error)
      //   err = { type: 'ERROR_EMPTY', input: '' }
      // else if (!is(Object, error))
      //   err = { message: error }
      // else err = error
      netReq.defer.reject(err);

      if (!netReq.options.noErrorBox) {
        //TODO weird code. `error` changed after `.reject`?

        /*err.input = method
         err.stack =
          stack ||
          hasPath(['originalError', 'stack'], error) ||
          error.stack ||
          (new Error()).stack*/
        _this5.emit('error.invoke', error);
      }
    }

    return _asyncToGenerator(function* () {
      netReq.options.requestID = netReq.requestID;
      _this5.emit('new-request', Object.assign({}, netReq));
      var rejectPromise = _ref3;

      function _ref4(waitTime) {
        return delayedCall(_req.performRequest, +waitTime * 1e3);
      }

      function _ref5() {
        return networker;
      }

      function _ref6(networker) {
        _req.config.networker = networker;
        return _req.performRequest();
      }

      function _ref7(error) {
        var deferResolve = netReq.defer.resolve;
        var apiSavedNet = _ref5;
        var apiRecall = _ref6;
        console.error(dTime(), 'Error', error.code, error.type, baseDcID, _dcID);

        return switchErrors(error, netReq.options, _dcID, baseDcID)(error, netReq.options, _dcID, _this5.emit, rejectPromise, requestThunk, apiSavedNet, apiRecall, deferResolve, _this5.mtpInvokeApi, _this5.storage);
      }

      try {
        yield _this5.initConnection();

        var requestThunk = _ref4;

        var _dcID = netReq.options.dcID ? netReq.options.dcID : yield _this5.storage.get('dc');

        var networker = yield _this5.mtpGetNetworker(_dcID);

        var cfg = {
          networker,
          dc: _dcID,
          storage: _this5.storage,
          getNetworker: _this5.mtpGetNetworker,
          netOpts: netReq.options
        };
        var _req = new Request(cfg, netReq.data.method, netReq.data.params);

        _req.performRequest().then(netReq.defer.resolve, _ref7).catch(rejectPromise);
      } catch (e) {
        netReq.defer.reject(e);
      }
      return netReq.defer.promise;
    })();
  }

}

var isAnyNetworker = ctx => Object.keys(ctx.cache.downloader).length > 0;

var netError = error => {
  console.log('Get networker error', error, error.stack);
  return Promise.reject(error);
};
//# sourceMappingURL=index.js.map