'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ApiManager = undefined;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _propEq = require('ramda/src/propEq');

var _propEq2 = _interopRequireDefault(_propEq);

var _has = require('ramda/src/has');

var _has2 = _interopRequireDefault(_has);

var _mtprotoLogger = require('mtproto-logger');

var _mtprotoLogger2 = _interopRequireDefault(_mtprotoLogger);

var _authorizer = require('../authorizer');

var _authorizer2 = _interopRequireDefault(_authorizer);

var _defer = require('../../util/defer');

var _defer2 = _interopRequireDefault(_defer);

var _dcConfigurator = require('../dc-configurator');

var _rsaKeysManger = require('../rsa-keys-manger');

var _rsaKeysManger2 = _interopRequireDefault(_rsaKeysManger);

var _error = require('../../error');

var _bin = require('../../bin');

var _errorCases = require('./error-cases');

var _mtprotoShared = require('mtproto-shared');

var _request = require('./request');

var _request2 = _interopRequireDefault(_request);

var _configProvider = require('../../config-provider');

var _configProvider2 = _interopRequireDefault(_configProvider);

var _networker = require('../networker');

var _networker2 = _interopRequireDefault(_networker);

var _request3 = require('../main/request');

var _request4 = _interopRequireDefault(_request3);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new _bluebird2.default(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return _bluebird2.default.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }
// import UpdatesManager from '../updates'

var debug = _mtprotoLogger2.default`api-manager`;

var baseDcID = 2;

var Ln = (length, obj) => {
  if (obj == null) return false;
  //$FlowIssue
  var res = (0, _propEq2.default)('length', length, obj);
  return res;
};

class ApiManager {
  constructor(config, uid) {
    var _this = this;

    this.cache = {
      uploader: {},
      downloader: {},
      auth: {},
      servers: {},
      keysParsed: {}
    };
    this.authPromise = (0, _defer2.default)();
    this.authBegin = false;
    this.currentDc = 2;
    this.online = false;

    function* _ref2(dcID, userAuth) {
      var fullUserAuth = Object.assign({ dcID }, userAuth);
      yield _this.storage.set('dc', dcID);
      yield _this.storage.set('user_auth', fullUserAuth);
      _this.emit('auth.dc', { dc: dcID, auth: userAuth });
    }

    this.setUserAuth = (() => {
      var _ref = _asyncToGenerator(_ref2);

      return function (_x, _x2) {
        return _ref.apply(this, arguments);
      };
    })();

    var {
      server,
      api,
      app: {
        storage,
        publicKeys
      }
    } = config;
    this.uid = uid;
    this.apiConfig = api;
    this.publicKeys = publicKeys;
    this.storage = storage;
    this.serverConfig = server;
    this.chooseServer = (0, _dcConfigurator.chooseServer)(this.cache.servers, server);
    var emitter = _configProvider2.default.rootEmitter(this.uid);
    this.on = emitter.on;
    this.emit = emitter.emit;
    this.keyManager = (0, _rsaKeysManger2.default)(uid, publicKeys, this.cache.keysParsed);
    this.auth = (0, _authorizer2.default)(uid, this.keyManager);

    //$FlowIssue
    this.mtpInvokeApi = this.mtpInvokeApi.bind(this);
    //$FlowIssue
    this.mtpGetNetworker = this.mtpGetNetworker.bind(this);

    // this.updatesManager = UpdatesManager(apiManager, this.TL)
    // apiManager.updates = this.updatesManager

    emitter.on('error.303', newDc => {
      this.authBegin = false;
      this.currentDc = newDc;
    });
  }
  networkSetter(dc, authKey, serverSalt) {
    var networker = new _networker2.default({
      appConfig: this.apiConfig,
      storage: this.storage,
      chooseServer: this.chooseServer
    }, dc, authKey, serverSalt, this.uid);
    this.cache.downloader[dc] = networker;
    return networker;
  }
  mtpGetNetworker(dcID) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      // const isUpload = options.fileUpload || options.fileDownload
      // const cache = isUpload
      //   ? this.cache.uploader
      //   : this.cache.downloader

      var cache = _this2.cache.downloader;
      if (!dcID) throw new Error('get Networker without dcID');

      if ((0, _has2.default)(dcID, cache)) return cache[dcID];

      var akk = `dc${dcID}_auth_key`;
      var ssk = `dc${dcID}_server_salt`;

      var dcUrl = _this2.chooseServer(dcID, false);
      if (typeof dcUrl !== 'string') throw new _error.DcUrlError(dcID, dcUrl);

      if (cache[dcID]) return cache[dcID];

      var authKeyHex = yield _this2.storage.get(akk);
      var serverSaltHex = yield _this2.storage.get(ssk);

      // if (!authKeyHex || !serverSaltHex)
      // await this.doAuth()

      if (Ln(512, authKeyHex)) {
        if (!serverSaltHex || serverSaltHex.length !== 16) serverSaltHex = 'AAAAAAAAAAAAAAAA';
        var _authKey = (0, _bin.bytesFromHex)(authKeyHex);
        var _serverSalt = (0, _bin.bytesFromHex)(serverSaltHex);

        return _this2.networkSetter(dcID, _authKey, _serverSalt);
      }

      var auth = void 0;
      try {
        auth = yield _this2.auth(dcID, _this2.cache.auth, dcUrl);
      } catch (error) {
        return netError(error);
      }

      var { authKey, serverSalt } = auth;

      yield _this2.storage.set(akk, (0, _bin.bytesToHex)(authKey));
      yield _this2.storage.set(ssk, (0, _bin.bytesToHex)(serverSalt));

      return _this2.networkSetter(dcID, authKey, serverSalt);
    })();
  }
  doAuth() {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      _this3.authBegin = true;
      try {
        var storedBaseDc = yield _this3.storage.get('dc');
        var baseDc = storedBaseDc || baseDcID;
        var opts = {
          dcID: baseDc,
          createNetworker: true
        };
        var networker = yield _this3.mtpGetNetworker(baseDc);
        var nearestDc = yield networker.wrapApiCall('help.getNearestDc', {}, opts);
        var { nearest_dc, this_dc } = nearestDc;
        if (storedBaseDc == null) {
          yield _this3.storage.set('dc', this_dc);
          // if (nearest_dc !== this_dc) await this.mtpGetNetworker(nearest_dc, {
          //   dcID           : this_dc,
          //   createNetworker: true
          // })
        }
        debug(`nearest Dc`, ` this dc`)(nearestDc, this_dc);
        _this3.emit('testing', { nearestDc, this_dc });
        _this3.authPromise.resolve();
      } catch (err) {
        _this3.authPromise.reject(err);
      }
    })();
  }
  initConnection() {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      if (!isAnyNetworker(_this4)) {
        if (!_this4.authBegin) _this4.doAuth();
        yield _this4.authPromise.promise;
      }
    })();
  }
  mtpInvokeApi(method, params, options = {}) {
    var deferred = (0, _defer2.default)();
    var netReq = new _request4.default({ method, params }, options, deferred);

    return this.invokeNetRequest(netReq);
  }

  invokeNetRequest(netReq) {
    var _this5 = this;

    function _ref3(error) {
      var err = void 0;
      if (error instanceof Error) err = error;else {
        err = new Error();
        //$FlowIssue
        err.data = error;
      }
      // if (!error)
      //   err = { type: 'ERROR_EMPTY', input: '' }
      // else if (!is(Object, error))
      //   err = { message: error }
      // else err = error
      netReq.defer.reject(err);

      if (!netReq.options.noErrorBox) {
        //TODO weird code. `error` changed after `.reject`?

        /*err.input = method
         err.stack =
          stack ||
          hasPath(['originalError', 'stack'], error) ||
          error.stack ||
          (new Error()).stack*/
        _this5.emit('error.invoke', error);
      }
    }

    return _asyncToGenerator(function* () {
      netReq.options.requestID = netReq.requestID;
      _this5.emit('new-request', Object.assign({}, netReq));
      var rejectPromise = _ref3;

      function _ref4(waitTime) {
        return (0, _mtprotoShared.delayedCall)(_req.performRequest, +waitTime * 1e3);
      }

      function _ref5() {
        return networker;
      }

      function _ref6(networker) {
        _req.config.networker = networker;
        return _req.performRequest();
      }

      function _ref7(error) {
        var deferResolve = netReq.defer.resolve;
        var apiSavedNet = _ref5;
        var apiRecall = _ref6;
        console.error((0, _mtprotoShared.dTime)(), 'Error', error.code, error.type, baseDcID, _dcID);

        return (0, _errorCases.switchErrors)(error, netReq.options, _dcID, baseDcID)(error, netReq.options, _dcID, _this5.emit, rejectPromise, requestThunk, apiSavedNet, apiRecall, deferResolve, _this5.mtpInvokeApi, _this5.storage);
      }

      try {
        yield _this5.initConnection();

        var requestThunk = _ref4;

        var _dcID = netReq.options.dcID ? netReq.options.dcID : yield _this5.storage.get('dc');

        var networker = yield _this5.mtpGetNetworker(_dcID);

        var cfg = {
          networker,
          dc: _dcID,
          storage: _this5.storage,
          getNetworker: _this5.mtpGetNetworker,
          netOpts: netReq.options
        };
        var _req = new _request2.default(cfg, netReq.data.method, netReq.data.params);

        _req.performRequest().then(netReq.defer.resolve, _ref7).catch(rejectPromise);
      } catch (e) {
        netReq.defer.reject(e);
      }
      return netReq.defer.promise;
    })();
  }

}

exports.ApiManager = ApiManager;
var isAnyNetworker = ctx => Object.keys(ctx.cache.downloader).length > 0;

var netError = error => {
  console.log('Get networker error', error, error.stack);
  return _bluebird2.default.reject(error);
};
//# sourceMappingURL=index.js.map